﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.IO;
using System.Drawing.Printing;
using System.Drawing.Drawing2D;

namespace Kresleni
{
    public partial class HlavniOkno : Form
    {
        private enum Tvar { Linka, Elipsa, Obdelnik, Volny, Zadny };

        private Tvar tvar;
        private bool kresba, zmena;
        private int zacX, zacY, konX, konY;
        private Bitmap bmp;
        private string soubor, pocAdresar;
        private Color barvaPera;
        private DashStyle stylPera;
        private int tloustkaPera;
        private int strana;

        public HlavniOkno()
        {
            InitializeComponent();

            soubor = Program.soubor;
            Text = "Kreslení - " + soubor;
            kresba = false;
            zmena = false;
            tvar = Tvar.Zadny;
            barvaPera = Color.Black;
            tloustkaPera = 1;
            stylPera = DashStyle.Solid;
            cmbTloustkaPera.SelectedIndex = 0;
            cmbStylPera.SelectedIndex = 0;
            cmbStylPera.ComboBox.DrawMode = DrawMode.OwnerDrawFixed;
            cmbStylPera.ComboBox.DrawItem += cmbStylPera_DrawItem;

            bmp = new Bitmap(pcbObrazek.ClientSize.Width,
                             pcbObrazek.ClientSize.Height,
                             PixelFormat.Format32bppArgb);

            using (Graphics gBmp = Graphics.FromImage(bmp))
            {
                gBmp.Clear(Color.White);
                pcbObrazek.Image = bmp;
            }

            if (soubor != "Beze jména")
               {
                   pocAdresar = Path.GetDirectoryName(soubor);
                   OtevriSoubor(soubor);
               }
               else pocAdresar = Application.StartupPath;

            odlgOtevrit.InitialDirectory = pocAdresar;
            sdlgUlozit.InitialDirectory = pocAdresar;
        }

        private void mniLinka_Click(object sender, EventArgs e)
        {
            tvar = Tvar.Linka;
            pcbObrazek.Cursor = Cursors.Cross;
        }

        private void mniObdelnik_Click(object sender, EventArgs e)
        {
            tvar = Tvar.Obdelnik;
            pcbObrazek.Cursor = Cursors.Cross;
        }

        private void mniElipsa_Click(object sender, EventArgs e)
        {
            tvar = Tvar.Elipsa;
            pcbObrazek.Cursor = Cursors.Cross;
        }

        private void mniVolne_Click(object sender, EventArgs e)
        {
            tvar = Tvar.Volny;
            pcbObrazek.Cursor = Cursors.Cross;
        }

        private void pcbObrazek_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left && tvar != Tvar.Zadny)
               {
                   kresba = true;
                   zacX = e.X; 
                   zacY = e.Y;
               }
        }

        private void ZacKonec(ref int x1, ref int y1, ref int x2, ref int y2)
        {
            int pX, pY, kX, kY;

            pX = Math.Min(x1, x2);
            pY = Math.Min(y1, y2);
            kX = Math.Max(x1, x2);
            kY = Math.Max(y1, y2);

            x1 = pX; y1 = pY; x2 = kX; y2 = kY;
        }

        private void pcbObrazek_Paint(object sender, PaintEventArgs e)
        {
            int pX, pY, kX, kY;
            Graphics g = e.Graphics;

            using (Pen pero = new Pen(barvaPera, tloustkaPera))
            {
                pero.DashStyle = stylPera;
                
                switch (tvar)
                {
                    case Tvar.Linka:
                        g.DrawLine(pero, zacX, zacY, konX, konY);
                        break;
                    case Tvar.Obdelnik:
                        pX = zacX; pY = zacY; kX = konX; kY = konY;
                        ZacKonec(ref pX, ref pY, ref kX, ref kY);
                        g.DrawRectangle(pero, pX, pY,
                                        kX - pX, kY - pY);
                        break;
                    case Tvar.Elipsa:
                        pX = zacX; pY = zacY; kX = konX; kY = konY;
                        ZacKonec(ref pX, ref pY, ref kX, ref kY);
                        g.DrawEllipse(pero, pX, pY,
                                      kX - pX, kY - pY);
                        break;
                }
            }
        }

        private void pcbObrazek_MouseMove(object sender, MouseEventArgs e)
        {
            if (!kresba)
            {
                tllSourad.Text = "[" + e.X + ", " + e.Y + "]";
            }
            else
            {
                konX = e.X; konY = e.Y;
                pcbObrazek.Refresh();
                tllSourad.Text = "[" + zacX + ", " + zacY + "] -> [" +
                                 konX + ", " + konY + "]";

                if (tvar == Tvar.Volny)
                    using (Graphics gBmp = Graphics.FromImage(bmp))
                    using (Pen pero = new Pen(barvaPera, tloustkaPera))
                    {
                        pero.DashStyle = stylPera;
                        gBmp.DrawLine(pero, zacX, zacY, konX, konY);
                        zacX = e.X; zacY = e.Y;
                    }
            }
        }

        private void pcbObrazek_MouseUp(object sender, MouseEventArgs e)
        {
            if (kresba)
            {
                using (Graphics gBmp = Graphics.FromImage(bmp))
                using (Pen pero = new Pen(barvaPera, tloustkaPera))
                {
                    pero.DashStyle = stylPera;

                    switch (tvar)
                    {
                        case Tvar.Linka:
                            gBmp.DrawLine(pero, zacX, zacY, konX, konY);
                            break;
                        case Tvar.Obdelnik:
                            ZacKonec(ref zacX, ref zacY, ref konX, ref konY);
                            gBmp.DrawRectangle(pero, zacX, zacY,
                                               konX - zacX, konY - zacY);
                            break;
                        case Tvar.Elipsa:
                            ZacKonec(ref zacX, ref zacY, ref konX, ref konY);
                            gBmp.DrawEllipse(pero, zacX, zacY,
                                            konX - zacX, konY - zacY);
                            break;
                    }
                }

                kresba = false;
                zmena = true;
                tvar = Tvar.Zadny;
                pcbObrazek.Cursor = Cursors.Arrow;
            }
        }

        private DialogResult UlozitZmeny()
        {
            DialogResult vysl;
 
            vysl = MessageBox.Show("Kresba v dokumentu " + 
                Path.GetFileName(soubor) +
                " byla změněna.\nChcete změny uložit?", "Uložit změny?",
                MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);

            if (vysl == DialogResult.Yes)
               mniUlozit_Click(null, null);

            return vysl;
        }

        private void pcbObrazek_ClientSizeChanged(object sender, EventArgs e)
        {
            if (bmp != null)
            {
                if (pcbObrazek.ClientSize.Width > bmp.Width)
                {
                    Bitmap nBmp = new Bitmap(pcbObrazek.ClientSize.Width,
                                             bmp.Height,
                                             bmp.PixelFormat);
                    using (Graphics gBmp = Graphics.FromImage(nBmp))
                    {
                        gBmp.Clear(Color.White);
                        gBmp.DrawImage(bmp, 0, 0);

                        bmp = nBmp;
                        pcbObrazek.Image = bmp;
                    }
                }
                if (pcbObrazek.ClientSize.Height > bmp.Height)
                {
                    Bitmap nBmp = new Bitmap(bmp.Width,
                                             pcbObrazek.ClientSize.Height,
                                             bmp.PixelFormat);
                    using (Graphics gBmp = Graphics.FromImage(nBmp))
                    {
                        gBmp.Clear(Color.White);
                        gBmp.DrawImage(bmp, 0, 0);

                        bmp = nBmp;
                        pcbObrazek.Image = bmp;
                    }
                }
            }
        }

        private void mniNovy_Click(object sender, EventArgs e)
        {
            DialogResult vysl;

            if (zmena)
               {
                  vysl = UlozitZmeny();
                  if (vysl != DialogResult.Cancel)
                     VytvorNovy();  
               }
               else VytvorNovy();
        }

        private void VytvorNovy()
        {
            using (Graphics gBmp = Graphics.FromImage(bmp))
            {
                gBmp.Clear(Color.White);
                pcbObrazek.Refresh();
            }
            zmena = false;
            soubor = "Beze jména";
            Text = "Kreslení - " + soubor;
        }

        private void mniOtevrit_Click(object sender, EventArgs e)
        {
            DialogResult vysl;

            if (zmena)
               {   
                  vysl = UlozitZmeny();
                  if (vysl != DialogResult.Cancel)
                     Otevri();
               }
               else Otevri();
        }

        private void Otevri()
        {
            if (odlgOtevrit.ShowDialog() == DialogResult.OK)
                OtevriSoubor(odlgOtevrit.FileName);
        }

        private void OtevriSoubor(string jmeno)
        {
            try
            {
                Bitmap tmpBmp = new Bitmap(jmeno);

                ClientSize = new Size(tmpBmp.Width,
                         tmpBmp.Height + hlavniMenu.Height +
                         stbStav.Height + tlbNastroje.Height);
                if (tmpBmp.PixelFormat != PixelFormat.Format32bppArgb)
                {
                    bmp = new Bitmap(tmpBmp.Width, tmpBmp.Height,
                                     PixelFormat.Format32bppArgb);
                    bmp.SetResolution(tmpBmp.HorizontalResolution,
                                      tmpBmp.VerticalResolution);
                    using (Graphics gBmp = Graphics.FromImage(bmp))
                    {
                        gBmp.DrawImage(tmpBmp, 0, 0);
                    }
                }
                else bmp = tmpBmp;
                pcbObrazek.Image = bmp;
                zmena = false;
                soubor = jmeno;
                Text = "Kreslení - " + Path.GetFileName(soubor);
            }
            catch (ArgumentException)
            {
                MessageBox.Show("Chybný formát souboru", "Chyba",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void mniUlozit_Click(object sender, EventArgs e)
        {
            if (soubor == "Beze jména")
               mniUlozitJako_Click(sender, e);
               else Uloz(soubor);
        }

        private void mniUlozitJako_Click(object sender, EventArgs e)
        {
            if (sdlgUlozit.ShowDialog() == DialogResult.OK)
            {
                soubor = sdlgUlozit.FileName;
                Uloz(soubor);
            }
        }

        private void Uloz(string soubor)
        {
            ImageFormat format;
            
            switch (Path.GetExtension(soubor).ToLower())
            {
                case ".jpg":
                case ".jpeg":
                    format = ImageFormat.Jpeg;
                    break;
                case ".bmp":
                    format = ImageFormat.Bmp;
                    break;
                case ".png":
                    format = ImageFormat.Png;
                    break;
                case ".gif":
                    format = ImageFormat.Gif;
                    break;
                case ".tif":
                case ".tiff":
                    format = ImageFormat.Tiff;
                    break;
                default:
                    format = ImageFormat.Jpeg;
                    break;
            }

            bmp.Save(soubor, format);
            Text = "Kreslení - " + Path.GetFileName(soubor);
            zmena = false;
        }

        private void mniZmenaBarvy_Click(object sender, EventArgs e)
        {
            cdlgBarva.Color = barvaPera;
            if (cdlgBarva.ShowDialog() == DialogResult.OK)
                barvaPera = cdlgBarva.Color;
        }

        private void mniKonec_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void HlavniOkno_FormClosing(object sender, FormClosingEventArgs e)
        {
            DialogResult vysl;
            
            if (zmena)
               {
                   vysl = UlozitZmeny();
                   if (vysl != DialogResult.Cancel)
                      e.Cancel = false;
                      else e.Cancel = true;
               }
        }

        private void mniOAplikaci_Click(object sender, EventArgs e)
        {
            OAplikaciDialog dlgOAplikaci = new OAplikaciDialog();

            dlgOAplikaci.ShowDialog();
        }

        private void mniTisk_Click(object sender, EventArgs e)
        {
            if (pdlgTisk.ShowDialog() == DialogResult.OK)
            {
                docTisk.DocumentName = Path.GetFileName(soubor);
                docTisk.OriginAtMargins = true;
                docTisk.DefaultPageSettings.Margins =
                    new Margins(42, 42, 42, 42);
                strana = 1;
                docTisk.Print();
            }
        }

        private void docTisk_PrintPage(object sender, PrintPageEventArgs e)
        {
            e.Graphics.DrawImage(bmp, 0, 0,
             new Rectangle(0, 0, bmp.Width, bmp.Height), GraphicsUnit.Pixel);
            if (strana < pdlgTisk.PrinterSettings.Copies)
            {
                strana++;
                e.HasMorePages = true;
            }
            else e.HasMorePages = false;
        }

        private void cmbStylPera_DrawItem(object sender, DrawItemEventArgs e)
        {
            Pen pero;

            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
               pero = new Pen(SystemColors.HighlightText);
               else pero = new Pen(SystemColors.MenuText);

            try
            {
                switch (e.Index)
                {
                    case 0:
                        pero.DashStyle = DashStyle.Solid;
                        break;
                    case 1:
                        pero.DashStyle = DashStyle.Dash;
                        break;
                    case 2:
                        pero.DashStyle = DashStyle.Dot;
                        break;
                    case 3:
                        pero.DashStyle = DashStyle.DashDot;
                        break;
                    case 4:
                        pero.DashStyle = DashStyle.DashDotDot;
                        break;
                }

                e.DrawBackground();
                e.Graphics.DrawLine(pero, e.Bounds.Left + 5, e.Bounds.Top + 7,
                                    e.Bounds.Right - 5, e.Bounds.Top + 7);
                e.DrawFocusRectangle();
            }
            finally
            {
                pero.Dispose();
            }
        }

        private void cmbStylPera_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cmbStylPera.ComboBox.SelectedIndex)
            {
                case 0:
                    stylPera = DashStyle.Solid;
                    break;
                case 1:
                    stylPera = DashStyle.Dash;
                    break;
                case 2:
                    stylPera = DashStyle.Dot;
                    break;
                case 3:
                    stylPera = DashStyle.DashDot;
                    break;
                case 4:
                    stylPera = DashStyle.DashDotDot;
                    break;
            }
        }

        private void cmbTloustkaPera_SelectedIndexChanged(object sender, EventArgs e)
        {
            tloustkaPera = cmbTloustkaPera.ComboBox.SelectedIndex + 1;
        }

        private void HlavniOkno_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.Copy;
                else e.Effect = DragDropEffects.None;
        }

        private void HlavniOkno_DragDrop(object sender, DragEventArgs e)
        {
            DialogResult vysl;
            string[] soubory = (string[])e.Data.GetData(DataFormats.FileDrop);

            if (zmena)
            {
                vysl = UlozitZmeny();
                if (vysl != DialogResult.Cancel)
                    OtevriSoubor(soubory[0]);
            }
            else OtevriSoubor(soubory[0]); ;
        }
    }
}
